package com.displaynone.acss.ui.auth

import android.annotation.SuppressLint
import android.graphics.drawable.TransitionDrawable
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import com.displaynone.acss.R
import com.displaynone.acss.databinding.FragmentAuthBinding
import com.displaynone.acss.util.collectWithLifecycle
import com.displaynone.acss.util.navigateTo
import com.displaynone.acss.ui.auth.AuthViewModel.Action

class AuthFragment: Fragment(R.layout.fragment_auth) {
    private var _binding: FragmentAuthBinding? = null
    private val binding: FragmentAuthBinding get() = _binding!!

    private val viewModel: AuthViewModel by viewModels()



    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        _binding = FragmentAuthBinding.bind(view)
        viewModel.action.collectWithLifecycle(this) { action ->
            if (action is Action.GotoProfile) {
                blockLoginButton() // FIXME() При двойном нажатии вылетает с ошибкой
                navigateTo(view, R.id.action_authFragment_to_profileFragment)
            }
        }
        viewModel.errorState.collectWithLifecycle(this) { errorStatus ->
            errorStatus?.let {
                Toast.makeText(requireContext(), it, Toast.LENGTH_SHORT).show()
                binding.errorTitle.text = errorStatus
                binding.errorTitle.visibility = View.VISIBLE
            }
        }
        binding.next.setOnClickListener{
            onLoginButtonClicked(view)
        }
    }
    private fun blockLoginButton() {
        binding.next.isEnabled = false
    }
    private fun getPasswordValidError(password: String): String {
        if (password.length < 8) {
            return "LenError"
        }
        return "NoErrors"
    }

    private fun validatePasswordAndSetError(password: String) {
        val errorType = getPasswordValidError(password)

        when (errorType) {
            "LenError" -> {
                binding.password.error = getString(R.string.error_password_too_short)
            }

            "NoErrors" -> {
                binding.password.error = null
            }
        }
    }
    private fun getLoginValidError(username: String): String {
        val alf = "^[a-zA-Z0-9_]+$".toRegex()
        if (username.isEmpty()) {
            return "EmptyError"
        }
        if (username.length < 3) {
            return "LenError"
        }
        if (username[0].isDigit()) {
            return "StartsWithDigitError"
        }
        if (!alf.matches(username)) {
            return "InvalidCharactersError"
        }
        return "NoErrors"
    }
//    private fun isPasswordValid(password: String): Boolean {
//        return password.isNotEmpty() &&
//                password.length >= 8
//    }

//    private fun subscribe() {
//        viewModel.state.collectWhenStarted(this) { state ->
//            binding.login.setOnClickListener(this::onLoginButtonClicked)
//        }
//    }
private fun validateLoginAndSetError(username: String) {
    val errorType = getLoginValidError(username)

    when (errorType) {
        "EmptyError" -> {
            binding.login.error = getString(R.string.error_login_empty)
        }
        "LenError" -> {
            binding.login.error = getString(R.string.error_login_too_short)
        }
        "StartsWithDigitError" -> {
            binding.login.error = getString(R.string.error_login_starts_with_digit)
        }
        "InvalidCharactersError" -> {
            binding.login.error = getString(R.string.error_login_invalid_characters)
        }
        "NoErrors" -> {
            binding.login.error = null
        }
    }
}
    private fun onLoginButtonClicked(view: View) {
        val login = binding.login.text.toString()
        val password = binding.password.text.toString()
        if (getPasswordValidError(password) != "NoErrors") {


            val color = R.color.secondary
            binding.next.backgroundTintList = ContextCompat.getColorStateList(requireContext(), color)
            validatePasswordAndSetError(password)
        }else{
            val color = R.color.primary
            binding.next.backgroundTintList = ContextCompat.getColorStateList(requireContext(), color)
        }
//        if (login.isEmpty()) return
        if (getLoginValidError(login) != "NoErrors") {
            val color = R.color.secondary
            binding.next.backgroundTintList = ContextCompat.getColorStateList(requireContext(), color)
            validateLoginAndSetError(login)
        }else{
            val color = R.color.primary
            binding.next.backgroundTintList = ContextCompat.getColorStateList(requireContext(), color)

        }

        viewModel.login(login, password)
    }


}